
#include "baseGraph.h"



/***************
 *  BaseGraph  *
 ***************/


/** Constructor. */
BaseGraph::BaseGraph(int mVC) :
  nodeTree(0), vertexCount(0), edgeCount(0),
     activeVertexCount(0), activeEdgeCount(0), groupCount(0),
     activeGroupCount(0), maxVertexCount(mVC) {

  nodes = new BaseVertexNode*[maxVertexCount];
  int i = 0;
  for (; i<maxVertexCount; i++) {
    nodes[i] = 0;
  }

  groups = new GenericList<BaseVertexNode*>();

  return;
}


/** Destructor. */
BaseGraph::~BaseGraph() {

  int i = 0;
  for (; i<vertexCount; i++) {
    delete nodes[i];
  }

  if (nodeTree != 0) {
    delete nodeTree;
  }

  delete nodes;
  delete groups;

  return;
}


/**
 * Get vertex count.
 * @return int: number of vertices in the graph.
 */
int BaseGraph::getVertexCount() {

  return vertexCount;
}


/**
 * Get edge count.
 * @return int: number of edges in the graph.
 */
int BaseGraph::getEdgeCount() {

  return edgeCount;
}


/**
 * Get ordered Coordinate3D data for the vertices.
 * @return Coordinate3D**: array of Coordinate3D* corresponding to the vertices.
 */
Coordinate3D** BaseGraph::getCoords() {

  Coordinate3D** coords = new Coordinate3D*[vertexCount];
  BaseVertexNode* node = 0;

  int i = 0;
  for (; i<vertexCount; i++) {
    node = nodes[i];
    if ( node->isActive() == 1 ) {
      coords[i] = node->getCoordinate3D()->clone();
    }
  }

  return coords;
}


/**
 * Get ConnectionMatrix for the graph.
 * @return ConnectionMatrix*: information about the connectivity (edges) in the graph.
 */
ConnectionMatrix* BaseGraph::getConnectionMatrix() {

  ConnectionMatrix* conMat = new ConnectionMatrix(vertexCount);
  BaseVertexNode* node1 = 0;
  BaseVertexNode* node2 = 0;

  int i = 0;
  for (; i<vertexCount; i++) {
    node1 = nodes[i];
    if ( node1->isActive() == 1 ) {
      node2 = node1->getFirstVertex();
      while (node2 != 0) {
	if ( node1->isActive(node2) == 1 &&
	     node2->getId() > i ) {
          conMat->setValue(i,node2->getId(),0);
	}
	node2 = node1->getNextVertex();
      }
    }
  }

  return conMat;
}
 

/**
 * Add nodes to the graph.
 * @param coord Coordinate3D*: coordinate for new node.
 * @return int: 0 if not active, 1 if active.
 */
int BaseGraph::addVertex(Coordinate3D* coord) {

  if (vertexCount >= maxVertexCount) {
    return 0;
  }

  BaseVertexNode* tempNode = new BaseVertexNode(coord);
  tempNode->setId(vertexCount);
  nodes[vertexCount] = tempNode;
  
  if (nodeTree != 0) {
    nodeTree->insert(nodes[vertexCount]);
  }
  else {
    nodeTree = new BaseVertexNodeTree(nodes[vertexCount]);
  }

  vertexCount++;
  activeVertexCount++;

  return 1;
}


/**
 * Add edges to the graph.
 * @param coord1 Coordinate3D*: first coordinate for new edge.
 * @param coord2 Coordinate3D*: second coordinate for new edge.
 * @return int: 0 on fail, 1 on success.
 */
int BaseGraph::addEdge(Coordinate3D* coord1, Coordinate3D* coord2) {

  // if (coords are equal) {don't add edge from vertex to itself}
  if (coord1->equals(coord2)) {
    return 0;
  }

  BaseVertexNode* tempNode1 = nodeTree->retrieve(coord1);
  BaseVertexNode* tempNode2 = nodeTree->retrieve(coord2);

  if (tempNode1 != 0 && tempNode2 != 0) {
    int result = tempNode1->addVertex(tempNode2);
    if (result != 0) {
      result = tempNode2->addVertex(tempNode1);
    }
    if (result != 0) {
      edgeCount++;
      if (tempNode1->isActive() &&
	  tempNode2->isActive()) {
	activeEdgeCount++;
      }
    }
    return result;
  }

  return 0;
}


/**
 * Activate all nodes in the graph.
 * @return int: 0 if not active, 1 if active.
 */
int BaseGraph::activateAll() {

  int i = 0;
  for (; i<vertexCount; i++) {
    nodes[i]->activate();
  }

  return 1;
}


/**
 * Deactivate all nodes in the graph.
 * @return int: 0 if not active, 1 if active.
 */
int BaseGraph::deactivateAll() {

  int i = 0;
  for (; i<vertexCount; i++) {
    nodes[i]->deactivate();
  }

  return 1;
}


/**
 * Deactivate outlier edges.
 * @return int: 0 if not active, 1 if active.
 */
int BaseGraph::deactivateLongEdges() {

  BaseVertexNode* node1 = 0;
  BaseVertexNode* node2 = 0;
  Coordinate3D*  coord1 = 0;
  Coordinate3D*  coord2 = 0;
  float edgeLengthSum = 0;
  float edgeLengthMean = 0;
  float edgeLengthSumOfMeanDiffSquared = 0;
  float edgeLengthSD = 0;   // Standard deviation.
  float edgeLengthCutoff = 0;
  int edgeNum = 0;

  // Compute sum and mean.
  int i=0;
  for (; i<vertexCount; i++) {
    node1 = nodes[i];
    if ( node1->isActive() == 1 ) {
      coord1 = node1->getCoordinate3D();
      node2 = node1->getFirstVertex();
      while (node2 != 0) {
	//if ( node2->isActive() == 1 &&
	if ( node2->getId() > i ) {
	  coord2 = node2->getCoordinate3D();
	  edgeLengthSum += coord1->getDistanceTo(coord2);
	  edgeNum++;
	}
	node2 = node1->getNextVertex();
      }
    }
  }

  if (edgeNum != edgeCount) {
    printf("Error: BaseGraph::deactivateLongEdges: edgeNum != edgeCount\n");
  }

  edgeLengthMean = edgeLengthSum / edgeCount;

  // Compute standard deviation.
  float diffFromMean = 0;
  for (i=0; i<vertexCount; i++) {
    node1 = nodes[i];
    if ( node1->isActive() == 1 ) {
      coord1 = node1->getCoordinate3D();
      node2 = node1->getFirstVertex();
      while (node2 != 0) {
	//if ( node2->isActive() == 1 &&
	if ( node2->getId() > i ) {
	  coord2 = node2->getCoordinate3D();
	  diffFromMean = coord1->getDistanceTo(coord2) - edgeLengthMean;
	  edgeLengthSumOfMeanDiffSquared += diffFromMean * diffFromMean;
	}
	node2 = node1->getNextVertex();
      }
    }
  }

  edgeLengthSD = sqrt(edgeLengthSumOfMeanDiffSquared / (edgeCount-1));
  edgeLengthCutoff = edgeLengthMean + 3 * edgeLengthSD;

  printf("edgeLengthMean:   %3.2f\n",edgeLengthMean);
  printf("edgeLengthSD:     %3.2f\n",edgeLengthSD);
  printf("edgeLengthCutoff: %3.2f\n",edgeLengthCutoff);


  // Remove edges longer than 2 SDs.
  for (i=0; i<vertexCount; i++) {
    node1 = nodes[i];
    if ( node1->isActive() == 1 ) {
      coord1 = node1->getCoordinate3D();
      node2 = node1->getFirstVertex();
      while (node2 != 0) {
	//if ( node2->isActive() == 1 &&
	if ( node2->getId() > i ) {
	  coord2 = node2->getCoordinate3D();
	  if ( coord1->getDistanceTo(coord2) > edgeLengthCutoff ) {
	    node1->deactivateEdge(node2);
	    node2->deactivateEdge(node1);
	  }
	}
	node2 = node1->getNextVertex();
      }
    }
  }
  
  return 0;
}


/**
 * Update or initially set the node IDs.
 * @return int: 0 if not active, 1 if active.
 */
int BaseGraph::refreshIds() {

  return 0;
}


/**
 * Update or initially set the nodes' group IDs.
 * @return int: 0 if not active, 1 if active.
 */
int BaseGraph::refreshGroupIds() {

  return 0;
}


/**
 * Create and return a TriangleGraph based on this BaseGraph.
 * @return TriangleGraph*: graph where a triangle is a node
 */
TriangleGraph* BaseGraph::getTriangleGraph() {

  //printf("->BaseGraph::getTriangleGraph\n");
  BaseVertexNode* node1 = 0;
  BaseVertexNode* node2 = 0;
  //BaseVertexNode* node3 = 0;
  //BaseVertexNode* node4 = 0;
  //GenericList<BaseEdge*> list1 = new GenericList<BaseVertexNode*>();
  //GenericList<BaseEdge*> list2 = new GenericList<BaseVertexNode*>();
  GenericList<BaseVertexNode**>* triangleList = new GenericList<BaseVertexNode**>();
  //BaseEdge* edge1 = 0;
  //BaseEdge* edge2 = 0;

  int i=0;
  for (; i<vertexCount; i++) {
    node1 = nodes[i];
    node2 = node1->getFirstVertex();
    while (node2 != 0) {
      if (node2->getId() > i) {
	getSharedNeighbors(node1, node2, triangleList);
      }
      node2 = node1->getNextVertex();
    }
  }

  // Add all the triangles from triangleList to tGraph
  //printf("  triangleList->getLength(): %d\n",triangleList->getLength());
  TriangleGraph* tGraph = new TriangleGraph(triangleList->getLength());
  BaseVertexNode** triangle = triangleList->front();
  int k = 0;
  while (triangle != 0) {
    //printf("  k: %d\n",k);
    tGraph->addVertex(triangle[0],triangle[1],triangle[2]);
    delete triangle;
    triangle = triangleList->front();
    k++;
  }

  //tGraph->addEdges();

  //printf("<-BaseGraph::getTriangleGraph\n");
  return tGraph;
}


/**
 *
 *
 */
int BaseGraph::getSharedNeighbors(BaseVertexNode* node1,
				  BaseVertexNode* node2,
				  GenericList<BaseVertexNode**>* triangleList) {
  
  //printf("->BaseGraph::getSharedNeighbors\n");
  //printf("  node1->getId(): %d\n",node1->getId());
  //printf("  node2->getId(): %d\n",node2->getId());
  
  // Create pair of neighbor lists
  GenericList<BaseEdge*>* list1 = node1->getList()->clone();
  GenericList<BaseEdge*>* list2 = node2->getList()->clone();
  if (list1 == 0 || list2 == 0) {
    printf("  NO LIST\n");
    return 1;
  }
  BaseEdge* edge1 = list1->front();
  //BaseEdge* edge2 = list2->front();
  BaseEdge* edge2 = list2->getFirst();
  while (edge1 != 0 && edge2 != 0) {
    //printf("  in while 1\n");
    /*
    while ( edge2 != 0 &&
	    ( edge2->vertex->getId() < edge1->vertex->getId() ||
	      edge2->vertex->getId() < node2->getId() ) ) {
      edge2 = list2->front();
    }
    //printf("  in while 2\n");
    if (edge2 == 0) {
      break;
    }
    //printf("  in while 3\n");
    while ( edge1 != 0 &&
	    ( edge1->vertex->getId() < edge2->vertex->getId() ||
	      edge1->vertex->getId() < node2->getId() ) ) {
      edge1 = list1->front();
    }
    //printf("  in while 4\n");
    if (edge1 == 0) {
      break;
    }
    */

    while (edge2 != 0 &&
	   edge2->vertex != edge1->vertex) {
      edge2 = list2->getNext();
    }
    if (edge2 == 0) {
      edge2 = list2->getFirst();
      if (edge2 == 0) {
	printf("Error: BaseGraph::getSharedNeighbors: list2 is dead\n:");
	break;
      }
    }

    //printf("  in while 5\n");
    if (edge1->vertex == edge2->vertex &&
	edge2->vertex->getId() > node2->getId()) {
      BaseVertexNode** triangle = new BaseVertexNode*[3];
      triangle[0] = node1;
      triangle[1] = node2;
      triangle[2] = edge1->vertex;
      triangleList->push(triangle);
      //printf("  triangle: %d, %d, %d\n",node1->getId(),node2->getId(),edge1->vertex->getId());
    }
    edge1 = list1->front();
    //printf("  in while 6\n");
  }

  // Discard lists
  delete list1;
  delete list2;

  //printf("<-BaseGraph::getSharedNeighbors\n");
  return 1;
}


/**
 * Get the difference in distance between corresponding vertices from two graphs.  This should be used when the graphs have the same number of vertices, and they are tessellations of the same system in different states.
 * @param graph BaseGraph*: graph to which the comparison is made.
 * @return int: 0 on fail, 1 on success.
 */
int BaseGraph::compareVertexPositions(BaseGraph* graph) {

  if (vertexCount != graph->getVertexCount()) {
    printf("Error: BaseGraph::compareVertexPositions - graphs have different number of vertices: %d and %d\n",vertexCount,graph->getVertexCount());
    return 0;
  }

  Coordinate3D** graphCoords = graph->getCoords();
  Coordinate3D* coord1 = 0;
  Coordinate3D* coord2 = 0;
  int i=0;
  for (; i<vertexCount; i++) {
    coord1 = nodes[i]->getCoordinate3D();
    coord2 = graphCoords[i];
    nodes[i]->setValue(coord1->getDistanceTo(coord2));
  }

  // Must delete the cloned Coordinate3Ds from the other graph.
  for (i=0; i<vertexCount; i++) {
    delete graphCoords[i];
  }
  delete graphCoords;

  return 1;
}


/**
 * Read in a tcl tessellation file to build a corresponding graph.
 * @return int: 0 on fail, 1 on success.
 */
int BaseGraph::readTessellationScript(FILE* infile) {

  /*
    if (infile == NULL) {
    printf("<=getAtomCluster(): 0\n");
    return 0;
    }
    
    char* tempStr1 = new char[1024];
  */
  
  return 0;
}


/**
 * Print a tcl tessellation file for the active vertices and edges of this graph.
 * @param outfile FILE*: tcl tessellation file in which to write the graph.
 * @param colorByValue int: whether to color the tessellation by value or not.
 * @return int: 0 on fail, 1 on success.
 */
int BaseGraph::printTessellationScript(FILE* outfile, int colorByValue) {


  fprintf(outfile,"\n");
  fprintf(outfile,"set mol top\n");
  fprintf(outfile,"\n");

  BaseVertexNode* node1 = 0;
  BaseVertexNode* node2 = 0;
  Coordinate3D*  coord1 = 0;
  Coordinate3D*  coord2 = 0;

  if (colorByValue == 0) {
    int i=0;
    for (; i<vertexCount; i++) {
      //printf("getting node %d\n",i);
      node1 = nodes[i];
      if ( node1->isActive() == 1 ) {
        coord1 = node1->getCoordinate3D();
        //fprintf(outfile,"graphics $mol sphere {%3.2f %3.2f %3.2f} radius 0.2 resolution 10\n",coord1->getX(),coord1->getY(),coord1->getZ());
        fprintf(outfile,"graphics $mol point {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ());
        //printf("getting first vertex\n");
        node2 = node1->getFirstVertex();
        //printf("got first vertex\n");
        //if (node2 == 0) {
        //printf("node2 == 0\n");
        //}
        //else {
        //printf("node2 != 0\n");
        //}
        //printf("node2->isActive(): %d\n",node2->isActive());
        //printf("node2->getId(): %d\n",node2->getId());
        while (node2 != 0) {
	  if ( node1->isActive(node2) == 1 &&
	       node2->getId() > i ) {
	    //printf("in if\n");
	    coord2 = node2->getCoordinate3D();
	    //printf("got coord2\n");
	    //fprintf(outfile,"graphics $mol cylinder {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f} radius 0.1 resolution 6\n",coord1->getX(),coord1->getY(),coord1->getZ(),coord2->getX(),coord2->getY(),coord2->getZ());
	    fprintf(outfile,"graphics $mol line {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ(),coord2->getX(),coord2->getY(),coord2->getZ());
	  }
	  node2 = node1->getNextVertex();
        }
        fprintf(outfile,"\n");
      }
    }
  }
  // Color the graph by value.
  else {
    // Get min and max values for the entire set of vertices.
    // Scale vertex values from their location in [min,max] to [0,1]
    node1 = nodes[0];
    float currentValue1 = node1->getValue();
    float currentValue2 = 0.0;
    float minValue = currentValue1;
    float maxValue = currentValue1;
    int i = 1;
    for (; i<vertexCount; i++) {
      node1 = nodes[i];
      currentValue1 = node1->getValue();
      if (currentValue1 < minValue) {
        minValue = currentValue1;
      }
      if (currentValue1 > maxValue) {
        maxValue = currentValue1;
      }
    }
    float offsetValue = maxValue - minValue;
    fprintf(outfile,"\n");
    fprintf(outfile,"# min value: %4.3f, max value: %4.3f\n",minValue,maxValue);
    fprintf(outfile,"\n");
    // Write tessellation file.
    for (i=0; i<vertexCount; i++) {
      node1 = nodes[i];
      currentValue1 = node1->getValue();
      if (currentValue1 < minValue || currentValue1 > maxValue) {
        printf("Error: BaseGraph::printTessellationScript - currentValue out of bounds\n");
      }
      currentValue1 -= minValue;
      currentValue1 /= offsetValue;
      // Specific for VMD colors, which are in the range [17,1040] with 1024 colors.
      currentValue1 *= 1023;
      currentValue1 += 17;
      if ( node1->isActive() == 1 ) {
        coord1 = node1->getCoordinate3D();
        //   Flip the percentage so that VMD color range is [blue,red]
        fprintf(outfile,"graphics $mol color %4.0f\n",1040-currentValue1);
        fprintf(outfile,"graphics $mol point {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ());
        node2 = node1->getFirstVertex();
        while (node2 != 0) {
	  if ( node1->isActive(node2) == 1 &&
	       node2->getId() > i ) {
            currentValue2 = node2->getValue();
            currentValue2 -= minValue;
            currentValue2 /= offsetValue;
            // Specific for VMD colors, which are in the range [17,1040] with 1024 colors.
            currentValue2 *= 1023;
            currentValue2 += 17;
            currentValue2 = (currentValue1 + currentValue2) / 2;
	    coord2 = node2->getCoordinate3D();
            //   Flip the percentage so that VMD color range is [blue,red]
            fprintf(outfile,"graphics $mol color %4.0f\n",1040-currentValue2);
	    fprintf(outfile,"graphics $mol line {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ(),coord2->getX(),coord2->getY(),coord2->getZ());
	  }
	  node2 = node1->getNextVertex();
        }
        fprintf(outfile,"\n");
      }
    }
  }

  return 1;
}


/**
 * Print a tcl tessellation file for the active vertices and edges of this graph.
 * @param outfile FILE*: tcl tessellation file in which to write the graph.
 * @param colorByValue int: whether to color the tessellation by value or not.
 * @return int: 0 on fail, 1 on success.
 */
int BaseGraph::printTessellationScript2(FILE* outfile, int colorByValue) {


  fprintf(outfile,"\n");
  fprintf(outfile,"set mol top\n");
  fprintf(outfile,"\n");

  BaseVertexNode* node1 = 0;
  BaseVertexNode* node2 = 0;
  Coordinate3D*  coord1 = 0;
  Coordinate3D*  coord2 = 0;

  if (colorByValue == 0) {
    int i=0;
    for (; i<vertexCount; i++) {
      //printf("getting node %d\n",i);
      node1 = nodes[i];
      if ( node1->isActive() == 1 ) {
        coord1 = node1->getCoordinate3D();
        //fprintf(outfile,"graphics $mol sphere {%3.2f %3.2f %3.2f} radius 0.2 resolution 10\n",coord1->getX(),coord1->getY(),coord1->getZ());
        fprintf(outfile,"graphics $mol point {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ());
        //printf("getting first vertex\n");
        node2 = node1->getFirstVertex();
        //printf("got first vertex\n");
        //if (node2 == 0) {
        //printf("node2 == 0\n");
        //}
        //else {
        //printf("node2 != 0\n");
        //}
        //printf("node2->isActive(): %d\n",node2->isActive());
        //printf("node2->getId(): %d\n",node2->getId());
        while (node2 != 0) {
	  if ( node1->isActive(node2) == 1 &&
	       node2->getId() > i ) {
	    //printf("in if\n");
	    coord2 = node2->getCoordinate3D();
	    //printf("got coord2\n");
	    //fprintf(outfile,"graphics $mol cylinder {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f} radius 0.1 resolution 6\n",coord1->getX(),coord1->getY(),coord1->getZ(),coord2->getX(),coord2->getY(),coord2->getZ());
	    fprintf(outfile,"graphics $mol line {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ(),coord2->getX(),coord2->getY(),coord2->getZ());
	  }
	  node2 = node1->getNextVertex();
        }
        fprintf(outfile,"\n");
      }
    }
  }
  // Color the graph by value.
  else {
    // Get min and max values for the entire set of vertices.
    // Scale vertex values from their location in [min,max] to [0,1]
    node1 = nodes[0];
    float currentValue1 = node1->getValue();
    float currentValue2 = 0.0;
    float minValue = currentValue1;
    float maxValue = currentValue1;
    int i = 1;
    for (; i<vertexCount; i++) {
      node1 = nodes[i];
      currentValue1 = node1->getValue();
      if (currentValue1 < minValue) {
        minValue = currentValue1;
      }
      if (currentValue1 > maxValue) {
        maxValue = currentValue1;
      }
    }
    float offsetValue = maxValue - minValue;
    fprintf(outfile,"\n");
    fprintf(outfile,"# min value: %4.3f, max value: %4.3f\n",minValue,maxValue);
    fprintf(outfile,"\n");
    // Write tessellation file.
    for (i=0; i<vertexCount; i++) {
      node1 = nodes[i];
      currentValue1 = node1->getValue();
      if (currentValue1 < minValue || currentValue1 > maxValue) {
        printf("Error: BaseGraph::printTessellationScript - currentValue out of bounds\n");
      }
      currentValue1 -= minValue;
      currentValue1 /= offsetValue;
      // Specific for VMD colors, which are in the range [17,1040] with 1024 colors.
      currentValue1 *= 1023;
      currentValue1 += 17;
      if ( node1->isActive() == 1 ) {
        coord1 = node1->getCoordinate3D();
        //   Flip the percentage so that VMD color range is [blue,red]
        fprintf(outfile,"graphics $mol color %4.0f\n",1040-currentValue1);
        fprintf(outfile,"graphics $mol point {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ());
        node2 = node1->getFirstVertex();
        while (node2 != 0) {
	  if ( node1->isActive(node2) == 1 &&
	       node2->getId() > i ) {
            currentValue2 = node2->getValue();
            currentValue2 -= minValue;
            currentValue2 /= offsetValue;
            // Specific for VMD colors, which are in the range [17,1040] with 1024 colors.
            currentValue2 *= 1023;
            currentValue2 += 17;
            currentValue2 = (currentValue1 + currentValue2) / 2;
	    coord2 = node2->getCoordinate3D();
            //   Flip the percentage so that VMD color range is [blue,red]
            fprintf(outfile,"graphics $mol color %4.0f\n",1040-currentValue2);
	    fprintf(outfile,"graphics $mol line {%3.2f %3.2f %3.2f} {%3.2f %3.2f %3.2f}\n",coord1->getX(),coord1->getY(),coord1->getZ(),coord2->getX(),coord2->getY(),coord2->getZ());
	  }
	  node2 = node1->getNextVertex();
        }
        fprintf(outfile,"\n");
      }
    }
  }

  return 1;
}


/**
 * Print a tcl tessellation file for the active vertices and edges of this graph.
 * @param outfile FILE*: tcl tessellation file in which to write the graph.
 * @return int: 0 on fail, 1 on success.
 */
int BaseGraph::printTessellationScript(FILE* outfile) {

  return printTessellationScript(outfile, 0);
}


/**
 * DEBUG PROC. Print out of node state.
 */
int BaseGraph::print() {

  printf("BaseGraph\n");
  printf("  vertexCount:       %d\n",vertexCount);
  printf("  edgeCount:         %d\n",edgeCount);
  printf("  activeVertexCount: %d\n",activeVertexCount);
  printf("  activeEdgeCount:   %d\n",activeEdgeCount);
  printf("  groupCount:        %d\n",groupCount);
  printf("  activeGroupCount:  %d\n",activeGroupCount);
  printf("  maxVertexCount:    %d\n",maxVertexCount);

  int i = 0;
  for (; i<vertexCount; i++) {
    printf("Node: %d\n", i);
    nodes[i]->print();
  }

  return 1;
}
