
#include "baseVertexNode.h"

/********************
 *  BaseVertexNode  *
 ********************/

/** Constructor. */
BaseVertexNode::BaseVertexNode(Coordinate3D* c) :
  group(0), id(-1), active(1), marked(0),
     vertexCount(0), activeVertexCount(0), value(0.0) {

  coord = new Coordinate3D(c->getX(),c->getY(),c->getZ());
  vertices = new GenericList<BaseEdge*>();

  return;
}


/** Destructor. */
BaseVertexNode::~BaseVertexNode() {

  delete coord;
  delete vertices;

  return;
}


/**
 * Get first node.
 * @return BaseVertexNode*: first node, 0 on fail.
 */
BaseVertexNode* BaseVertexNode::getFirstVertex() {

  BaseEdge* tempEdge = vertices->getFirst();

  if (tempEdge == 0) {
    return 0;
  }

  return tempEdge->vertex;
}


/**
 * Get next node.
 * @return BaseVertexNode*: next node, 0 on fail.
 */
BaseVertexNode* BaseVertexNode::getNextVertex() {

  BaseEdge* tempEdge = vertices->getNext();

  if (tempEdge == 0) {
    return 0;
  }

  return tempEdge->vertex;
}


/**
 * Get the 3D coordinates.
 * @return Coordinate3D*: coordinates.
 */
Coordinate3D* BaseVertexNode::getCoordinate3D() {

  return coord;
}


/**
 * Get the list of neighboring nodes.
 * @return GenericList<BaseEdge*>*: list.
 */
GenericList<BaseEdge*>* BaseVertexNode::getList() {

  return vertices;
}


/**
 * Get ID.
 * @return int: ID.
 */
int BaseVertexNode::getId() {

  return id;
}


/**
 * Set Id.
 * @param i int: ID.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::setId(int i) {

  id = i;

  return 1;
}


/**
 * Get group ID.
 * @return GroupId*: group ID.
 */
GroupId* BaseVertexNode::getGroupId() {

  return group;
}


/**
 * Set group ID.
 * @param gId GroupId*: group ID.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::setGroupId(GroupId* gId) {
  
  group = gId;

  return 1;
}


/**
 * Get vertex count.
 * @return int: vertex count.
 */
int BaseVertexNode::getVertexCount() {

  return vertexCount;
}


/**
 * Get active vertex count.
 * @return int: active vertex count.
 */
int BaseVertexNode::getActiveVertexCount() {

  return activeVertexCount;
}


/**
 * Set node's value.
 * @param val float: number to be associated with the nodes' value.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::setValue(float val) {

  value = val;

  return 1;
}


/**
 * Get node's value.
 * @return float: value.
 */
float BaseVertexNode::getValue() {

  return value;
}


/**
 * Activate this node.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::activate() {

  active = 1;

  return 1;
}


/**
 * Deactivate this node.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::deactivate() {

  active = 0;

  BaseEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->active == 1) {
      tempEdge->vertex->deactivateEdge(this);
      tempEdge->active = 0;
      activeVertexCount--;
    }
    tempEdge = vertices->getNext();
  }

  return 1;
}


/**
 * Activate the edge leading to the specified node.
 * @param b BaseVertexNode*: node the edge connects to.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::activateEdge(BaseVertexNode* b) {

  if (isActive() && b->isActive()) {
    BaseEdge* tempEdge = vertices->getFirst();
    while (tempEdge != 0) {
      if (tempEdge->vertex == b) {
	tempEdge->active = 1;
	activeVertexCount++;
	return 1;
      }
      tempEdge = vertices->getNext();
    }
  }

  return 0;
}


/**
 * Deactivate the edge leading to the specified node.
 * @param b BaseVertexNode*: node the edge connects to.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::deactivateEdge(BaseVertexNode* b) {

  BaseEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->vertex == b) {
      tempEdge->active = 0;
      activeVertexCount--;
      return 1;
    }
    tempEdge = vertices->getNext();
  }

  return 0;
}


/**
 * Check whether this node is active or not.
 * @return int: 0 if not active, 1 if active.
 */
int BaseVertexNode::isActive() {

  return active;
}


/**
 * Check whether the edge leading to the specified node is active.
 * @param b BaseVertexNode*: node the edge connects to.
 * @return int: 0 if not active, 1 if active.
 */
int BaseVertexNode::isActive(BaseVertexNode* b) {

  BaseEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->vertex == b) {
      return tempEdge->active;
    }
    tempEdge = vertices->getNext();
  }

  return 0;
}


/**
 * DEBUG PROC. Print out of node state.
 */
int BaseVertexNode::print() {

  int g = 0;
  if (group != 0) {
    g = group->getId();
  }

  printf("BaseVertexNode:\n");
  printf("  coord: (%3.2f,%3.2f,%3.2f)\n", coord->getX(), coord->getY(), coord->getZ());
  printf("  group: %d\n", g);
  printf("  id: %d\n", id);
  printf("  active: %d\n", active);
  printf("  marked: %d\n", marked);
  printf("  vertexCount: %d\n", vertexCount);
  printf("  activeVertexCount: %d\n", activeVertexCount);
  printf("  value: %3.2f\n",value);
  printf("  vertices: ");
  BaseEdge* tempEdge = vertices->getFirst();
  do {
    if ( tempEdge != 0) {
      printf("%d ",tempEdge->vertex->getId());
      tempEdge = vertices->getNext();
    }
  } while ( tempEdge != 0 );
  printf("\n");
  printf("  active vertices: ");
  tempEdge = vertices->getFirst();
  do {
    if ( tempEdge != 0) {
      if ( tempEdge->active == 1) {
	printf("%d ",tempEdge->vertex->getId());
      }
      tempEdge = vertices->getNext();
    }
  } while ( tempEdge != 0 );
  printf("\n");

  return 1;
}


/**
 * Add a vertex with an edge to this node.
 * @param b BaseVertexNode*: node to add.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::addVertex(BaseVertexNode* b) {

  BaseEdge* edge = new BaseEdge(b);
  vertices->push(edge);
  vertexCount++;
  if (isActive() && b->isActive()) {
    activeVertexCount++;
  }
  else {
    edge->active = 0;
  }

  return 1;
}


/**
 * Mark this node.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::mark() {

  marked = 1;

  return 1;
}


/**
 * Unmark this node.
 * @return int: 0 on fail, 1 on success.
 */
int BaseVertexNode::unmark() {

  marked = 0;

  return 1;
}


/**
 * Check whether this node is marked or not.
 * @return int: 0 if not active, 1 if active.
 */
int BaseVertexNode::isMarked() {

  return marked;
}
