
#ifndef BASEVERTEXNODE_H
#define BASEVERTEXNODE_H

#include "coordinate3D.h"
#include "genericList.h"
#include "genericList.c"
#include "groupId.c"

#include <stdio.h>

class BaseEdge;


/**
 * Nodes for BaseGraph.
 */
class BaseVertexNode {



 public:
  /** Constructor. */
  BaseVertexNode(Coordinate3D* c);

  /** Destructor. */
  ~BaseVertexNode();

  /**
   * Get first node.
   * @return BaseVertexNode*: first node, 0 on fail.
   */
  BaseVertexNode* getFirstVertex();

  /**
   * Get next node.
   * @return BaseVertexNode*: next node, 0 on fail.
   */
  BaseVertexNode* getNextVertex();

  /**
   * Get the 3D coordinates.
   * @return Coordinate3D*: coordinates.
   */
  Coordinate3D* getCoordinate3D();

  /**
   * Get the list of neighboring nodes.
   * @return GenericList<BaseEdge*>*: list.
   */
  GenericList<BaseEdge*>* getList();

  /**
   * Get ID.
   * @return int: ID.
   */
  int getId();

  /**
   * Set ID.
   * @param i int: ID.
   * @return int: 0 on fail, 1 on success.
   */
  int setId(int i);

  /**
   * Get group ID.
   * @return GroupId*: group ID.
   */
  GroupId* getGroupId();

  /**
   * Set group ID.
   * @param gId GroupId*: group ID.
   * @return int: 0 on fail, 1 on success.
   */
  int setGroupId(GroupId* gId);

  /**
   * Get vertex count.
   * @return int: vertex count.
   */
  int getVertexCount();

  /**
   * Get active vertex count.
   * @return int: active vertex count.
   */
  int getActiveVertexCount();

  /**
   * Set node's value.
   * @param val float: number to be associated with the nodes' value.
   * @return int: 0 on fail, 1 on success.
   */
  int setValue(float val);

  /**
   * Get node's value.
   * @return float: value.
   */
  float getValue();

  /**
   * Activate this node.
   * @return int: 0 on fail, 1 on success.
   */
  int activate();

  /**
   * Deactivate this node.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivate();

  /**
   * Activate the edge leading to the specified node.
   * @param b BaseVertexNode*: node the edge connects to.
   * @return int: 0 on fail, 1 on success.
   */
  int activateEdge(BaseVertexNode* b);

  /**
   * Deactivate the edge leading to the specified node.
   * @param b BaseVertexNode*: node the edge connects to.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateEdge(BaseVertexNode* b);

  /**
   * Check whether this node is active or not.
   * @return int: 0 if not active, 1 if active.
   */
  int isActive();

  /**
   * Check whether the edge leading to the specified node is active.
   * @param b BaseVertexNode*: node the edge connects to.
   * @return int: 0 if not active, 1 if active.
   */
  int isActive(BaseVertexNode* b);

  /**
   * DEBUG PROC. Print out of node state.
   */
  int print();


  //protected:
  /**
   * Add a vertex with an edge to this node.
   * @param b BaseVertexNode*: node to add.
   * @return int: 0 on fail, 1 on success.
   */
  int addVertex(BaseVertexNode* b);
  
  /**
   * Mark this node.
   * @return int: 0 on fail, 1 on success.
   */
  int mark();

  /**
   * Unmark this node.
   * @return int: 0 on fail, 1 on success.
   */
  int unmark();

  /**
   * Check whether this node is marked or not.
   * @return int: 0 if not active, 1 if active.
   */
  int isMarked();

 private:
  Coordinate3D* coord;
  GenericList<BaseEdge*>* vertices;
  GroupId* group;
  int id;
  int active;
  int marked;
  int vertexCount;
  int activeVertexCount;
  float value;

};


/**
 * Edges for BaseVertexNode.
 */
class BaseEdge {
 public:
  BaseVertexNode* vertex;
  int active;
  
  BaseEdge(BaseVertexNode* v) :
    vertex(v), active(1) {}
};

#endif
