
#include "tetrahedronGraphNode.h"


/** Constructor. */
TetrahedronGraphNode::TetrahedronGraphNode(TriangleGraphNode* node1,
					   TriangleGraphNode* node2,
					   TriangleGraphNode* node3,
					   TriangleGraphNode* node4) :
  group(0), id(-1), active(1), marked(0),
     vertexCount(0), activeVertexCount(0) {

  //printf("->TetrahedronGraphNode::Constructor\n");
  tetrahedron = new TetrahedronNode(node1, node2, node3, node4);
  vertices = new GenericList<TetrahedronGraphEdge*>();

  //printf("<-TetrahedronGraphNode::Constructor\n");
  return;
}

/** Destructor. */
TetrahedronGraphNode::~TetrahedronGraphNode() {

  delete tetrahedron;
  delete vertices;

  return;
}


/**
 * Get first node.
 * @return TetrahedronGraphNode*: first node, 0 on fail.
 */
TetrahedronGraphNode* TetrahedronGraphNode::getFirstVertex() {

  TetrahedronGraphEdge* tempEdge = vertices->getFirst();

  if (tempEdge == 0) {
    return 0;
  }

  return tempEdge->vertex;
}


/**
 * Get next node.
 * @return TetrahedronGraphNode*: next node, 0 on fail.
 */
TetrahedronGraphNode* TetrahedronGraphNode::getNextVertex() {

  TetrahedronGraphEdge* tempEdge = vertices->getNext();

  if (tempEdge == 0) {
    return 0;
  }

  return tempEdge->vertex;
}


/**
 * Get the 3D coordinates.
 * @param i int: coordinate 0, 1, 2, or 3.
 * @return Coordinate3D*: tetrahedron corner.
 */
Coordinate3D* TetrahedronGraphNode::getCoordinate3D(int i) {

  if (i == 0) {
    return tetrahedron->vertex1;
  }
  if (i == 1) {
    return tetrahedron->vertex2;
  }
  if (i == 2) {
    return tetrahedron->vertex3;
  }
  if (i == 3) {
    return tetrahedron->vertex4;
  }

  printf("Error TetrahedronGraphNode::getCoordinate3D - index %d out of bounds\n", i);

  return 0;
}


/**
 * Get the 3D coordinates.
 * @param i int: face 0, 1, 2, or 3.
 * @param j int: index 0, 1, or 2.
 * @return Coordinate3D*: triangle corner.
 */
Coordinate3D* TetrahedronGraphNode::getCoordinate3D(int i, int j) {

  if (i == 0) {
    return tetrahedron->node1->getCoordinate3D(j);
  }
  if (i == 1) {
    return tetrahedron->node2->getCoordinate3D(j);
  }
  if (i == 2) {
    return tetrahedron->node3->getCoordinate3D(j);
  }
  if (i == 3) {
    return tetrahedron->node4->getCoordinate3D(j);
  }

  printf("Error TetrahedronGraphNode::getCoordinate3D - index %d out of bounds\n", i);

  return 0;
}


/**
 * Get the triangular tetrahedron face.
 * @param i int: face 0, 1, 2, or 3.
 * @return TriangleGraphNode*: face.
 */
TriangleGraphNode* TetrahedronGraphNode::getFace(int i) {

  //printf("->TetrahedronGraphNode::getFace\n");

  if (i == 0) {
    //printf("<-TetrahedronGraphNode::getFace\n");
    return tetrahedron->node1;
  }
  if (i == 1) {
    //printf("<-TetrahedronGraphNode::getFace\n");
    return tetrahedron->node2;
  }
  if (i == 2) {
    //printf("<-TetrahedronGraphNode::getFace\n");
    return tetrahedron->node3;
  }
  if (i == 3) {
    //printf("<-TetrahedronGraphNode::getFace\n");
    return tetrahedron->node4;
  }

  printf("Error TetrahedronGraphNode::getFace index out of bounds\n");

  return 0;
}


/**
 * Get the value of the node.
 * @param i int: face 0, 1, or 2, or 3.
 * @param j int: index 0, 1, or 2.
 * @return float: value.
 */
float TetrahedronGraphNode::getValue(int i, int j) {

  if (i == 0) {
    return tetrahedron->node1->getValue(j);
  }
  if (i == 1) {
    return tetrahedron->node2->getValue(j);
  }
  if (i == 2) {
    return tetrahedron->node3->getValue(j);
  }
  if (i == 3) {
    return tetrahedron->node4->getValue(j);
  }

  printf("Error TetrahedronGraphNode::getValue - index %d out of bounds\n", i);

  return 0;
}


/**
 * Get ID.
 * @return int: ID.
 */
int TetrahedronGraphNode::getId() {

  return id;
}


/**
 * Set ID.
 * @param i int: ID.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::setId(int i) {

  id = i;

  return 1;
}


/**
 * Get group ID.
 * @return GroupId*: group ID.
 */
GroupId* TetrahedronGraphNode::getGroupId() {

  return group;
}


/**
 * Set group ID.
 * @param gId GroupId*: group ID.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::setGroupId(GroupId* gId) {

  group = gId;

  return 1;
}


/**
 * Get vertex count.
 * @return int: vertex count.
 */
int TetrahedronGraphNode::getVertexCount() {

  return vertexCount;
}


/**
 * Get active vertex count.
 * @return int: active vertex count.
 */
int TetrahedronGraphNode::getActiveVertexCount() {

  return activeVertexCount;
}


/**
 * Set "type" attribute of internal triangles to INTERNAL.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::setInternalTriangleTypes() {

  //printf("->TetrahedronGraphNode::setInternalTriangleTypes\n");

  int i = 0;
  //int j = 0;
  //int k = 0;
  int result = 0;
  TetrahedronGraphEdge* neighborEdge = vertices->getFirst();
  TetrahedronGraphNode* neighbor = 0;
  TriangleGraphNode* triangle = 0;
  while (neighborEdge != 0) {
    neighbor = neighborEdge->vertex;
    if (neighbor->getId() > getId()) {
      triangle = getSharedFace(neighbor);
      if (triangle != 0) {
	//printf("  INTERNAL: %d, %d; triangle Id: %d\n",getId(),neighbor->getId(),triangle->getId());
	result += triangle->setType(TriangleGraphNode::INTERNAL);
      }
      else {
	printf("Error TetrahedronGraphNode::setInternalTriangleTypes no shared face\n");
      }
    }
    i++;
    neighborEdge = vertices->getNext();
  }

  if (i>4) {
    printf("Error TetrahedronGraphNode::setTriangleTypes too many faces for a tetrahedron: %d\n",i);
  }

  if (result > 0) {
    result = 1;
  }

  //printf("<-TetrahedronGraphNode::setInternalTriangleTypes\n");

  return result;
}


/**
 * Get face shared by two neighboring tetrahedra.
 * @param tetraNode TetrahedronGraphNode*: node this tetrahedron connects to.
 * @return TriangleGraphNode*: face that the two tetrahedra share.
 */
TriangleGraphNode* TetrahedronGraphNode::getSharedFace(TetrahedronGraphNode* tetraNode) {

  //printf("->TetrahedronGraphNode::getSharedFace\n");

  int i=0;
  int j=0;
  for (i=0; i<4; i++) {
    for (j=0; j<4; j++) {
      //printf("  id1: %d, id2: %d, (%d,%d)\n",getId(),tetraNode->getId(),i,j);
      if (getFace(i) == tetraNode->getFace(j)) {
	return getFace(i);
      }
    }
  }

  //printf("<-TetrahedronGraphNode::getSharedFace\n");

  return 0;
}


/**
 * Get volume.
 * @return float: volume of this tetrahedron.
 */
float TetrahedronGraphNode::getVolume() {

  // Volume = (1/3!)*(a.(b x c)) where a,b,c are edge vectors
  //    pointing away from the same vertex.

  float volume = 0.0;
  Coordinate3D** coords = new Coordinate3D*[4];
  Coordinate3D** edges = new Coordinate3D*[3];

  coords[0] = getCoordinate3D(0,0);
  coords[1] = getCoordinate3D(0,1);
  coords[2] = getCoordinate3D(0,2);

  int i=0;
  for (i=0; i<3; i++) {
    if (!tetrahedron->node1->hasCoordinate3D(getCoordinate3D(1,i))) {
      coords[3] = getCoordinate3D(1,i);
      break;
    }
  }

  edges[0] = coords[1]->minus(coords[0]);
  edges[1] = coords[2]->minus(coords[0]);
  edges[2] = coords[3]->minus(coords[0]);

  volume = edges[0]->dot(edges[1]->cross(edges[2]));
  volume /= 6.0;

  if (volume < 0.0) {
    volume *= -1.0;
  }

  return volume;
}


/**
 * Get volume.
 * @return float: volume of this tetrahedron.
 */
float TetrahedronGraphNode::getVolume2() {

  /* Volume = (1/3!)*det(x1 y1 z1 1)
                        (x2 y2 z2 1)
                        (x3 y3 z3 1)
                        (x4 y4 z4 1)
  */

  float volume = 0.0;
  float** matrix = new float*[4];

  int i=0;
  for (i=0; i<4; i++) {
    matrix[i] = new float[4];
  }

  for (i=0; i<3; i++) {
    matrix[i][0] = getCoordinate3D(0,i)->getX();
    matrix[i][1] = getCoordinate3D(0,i)->getY();
    matrix[i][2] = getCoordinate3D(0,i)->getZ();
    matrix[i][3] = 1.0;
  }

  for (i=0; i<3; i++) {
    if (!tetrahedron->node1->hasCoordinate3D(getCoordinate3D(1,i))) {
      //coords[3] = getCoordinate3D(1,i);
      matrix[3][0] = getCoordinate3D(1,i)->getX();
      matrix[3][1] = getCoordinate3D(1,i)->getY();
      matrix[3][2] = getCoordinate3D(1,i)->getZ();
      matrix[3][3] = 1.0;
      break;
    }
  }


  // XXX - DEBUG PRINTOUT
  /*
  printf("matrix:\n");
  for (i=0; i<4; i++) {
    printf(" [ %3.3f %3.3f %3.3f %3.3f ]\n",matrix[i][0],matrix[i][1],matrix[i][2],matrix[i][3]);
  }
  printf("\n");
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",matrix[0][0],matrix[1][1],matrix[2][2],matrix[3][3],(matrix[0][0] * matrix[1][1] * matrix[2][2] * matrix[3][3]));
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",matrix[0][0],matrix[1][1],matrix[2][3],matrix[3][2],(matrix[0][0] * matrix[1][1] * matrix[2][3] * matrix[3][2]));
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",matrix[0][0],matrix[1][2],matrix[2][1],matrix[3][3],(matrix[0][0] * matrix[1][2] * matrix[2][1] * matrix[3][3]));
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",matrix[0][0],matrix[1][2],matrix[2][3],matrix[3][1],(matrix[0][0] * matrix[1][2] * matrix[2][3] * matrix[3][1]));
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",matrix[0][0],matrix[1][3],matrix[2][1],matrix[3][2],(matrix[0][0] * matrix[1][3] * matrix[2][1] * matrix[3][2]));
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",matrix[0][0],matrix[1][3],matrix[2][2],matrix[3][1],(matrix[0][0] * matrix[1][3] * matrix[2][2] * matrix[3][1]));

  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  printf("%3.3f * %3.3f * %3.3f * %3.3f = %3.3f\n",);
  */

  // Calculate 4x4 determinant (24 terms).
  volume = (matrix[0][0] * matrix[1][1] * matrix[2][2] * matrix[3][3])
    - (matrix[0][0] * matrix[1][1] * matrix[2][3] * matrix[3][2])
    - (matrix[0][0] * matrix[1][2] * matrix[2][1] * matrix[3][3])
    + (matrix[0][0] * matrix[1][2] * matrix[2][3] * matrix[3][1])
    + (matrix[0][0] * matrix[1][3] * matrix[2][1] * matrix[3][2])
    - (matrix[0][0] * matrix[1][3] * matrix[2][2] * matrix[3][1])
    - (matrix[0][1] * matrix[1][0] * matrix[2][2] * matrix[3][3])
    + (matrix[0][1] * matrix[1][0] * matrix[2][3] * matrix[3][2])
    + (matrix[0][1] * matrix[1][2] * matrix[2][0] * matrix[3][3])
    - (matrix[0][1] * matrix[1][2] * matrix[2][3] * matrix[3][0])
    - (matrix[0][1] * matrix[1][3] * matrix[2][0] * matrix[3][2])
    + (matrix[0][1] * matrix[1][3] * matrix[2][2] * matrix[3][0])
    + (matrix[0][2] * matrix[1][0] * matrix[2][1] * matrix[3][3])
    - (matrix[0][2] * matrix[1][0] * matrix[2][3] * matrix[3][1])
    - (matrix[0][2] * matrix[1][1] * matrix[2][0] * matrix[3][3])
    + (matrix[0][2] * matrix[1][1] * matrix[2][3] * matrix[3][0])
    + (matrix[0][2] * matrix[1][3] * matrix[2][0] * matrix[3][1])
    - (matrix[0][2] * matrix[1][3] * matrix[2][1] * matrix[3][0])
    - (matrix[0][3] * matrix[1][0] * matrix[2][1] * matrix[3][2])
    + (matrix[0][3] * matrix[1][0] * matrix[2][2] * matrix[3][1])
    + (matrix[0][3] * matrix[1][1] * matrix[2][0] * matrix[3][2])
    - (matrix[0][3] * matrix[1][1] * matrix[2][2] * matrix[3][0])
    - (matrix[0][3] * matrix[1][2] * matrix[2][0] * matrix[3][1])
    + (matrix[0][3] * matrix[1][2] * matrix[2][1] * matrix[3][0]);
  volume /= 6.0;

  if (volume < 0.0) {
    volume *= -1.0;
  }

  for (i=0; i<4; i++) {
    delete matrix[i];
  }
  delete matrix;

  return volume;

  /*
    return matrix[0][1][2][3] - matrix[0][1][3][2]
    - matrix[0][2][1][3] + matrix[0][2][3][1]
    + matrix[0][3][1][2] - matrix[0][3][2][1]
    - matrix[1][0][2][3] + matrix[1][0][3][2]
    + matrix[1][2][0][3] - matrix[1][2][3][0]
    - matrix[1][3][0][2] + matrix[1][3][2][0]
    + matrix[2][0][1][3] - matrix[2][0][3][1]
    - matrix[2][1][0][3] + matrix[2][1][3][0]
    + matrix[2][3][0][1] - matrix[2][3][1][0]
    
    - matrix[3][0][1][2] + matrix[3][0][2][1]
    + matrix[3][1][0][2] - matrix[3][1][2][0]
    - matrix[3][2][0][1] + matrix[3][2][1][0];
  */
}


/**
 * Activate this node.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::activate() {

  active = 1;

  return 1;
}


/**
 * Deactivate this node.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::deactivate() {

  active = 0;

  TetrahedronGraphEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->active == 1) {
      tempEdge->vertex->deactivateEdge(this);
      tempEdge->active = 0;
      activeVertexCount--;
    }
    tempEdge = vertices->getNext();
  }

  return 1;
}


/**
 * Activate the edge leading to the specified node.
 * @param b TetrahedronGraphNode*: node the edge connects to.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::activateEdge(TetrahedronGraphNode* b) {

  if (isActive() && b->isActive()) {
    TetrahedronGraphEdge* tempEdge = vertices->getFirst();
    while (tempEdge != 0) {
      if (tempEdge->vertex == b) {
	tempEdge->active = 1;
	activeVertexCount++;
	return 1;
      }
      tempEdge = vertices->getNext();
    }
  }

  return 0;
}


/**
 * Deactivate the edge leading to the specified node.
 * @param b TetrahedronGraphNode*: node the edge connects to.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::deactivateEdge(TetrahedronGraphNode* b) {

  TetrahedronGraphEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->vertex == b) {
      tempEdge->active = 0;
      activeVertexCount--;
      return 1;
    }
    tempEdge = vertices->getNext();
  }

  return 0;
}


/**
 * Check whether this node is active or not.
 * @return int: 0 if not active, 1 if active.
 */
int TetrahedronGraphNode::isActive() {

  return active;
}


/**
 * Check whether the edge leading to the specified node is active.
 * @param b TetrahedronGraphNode*: node the edge connects to.
 * @return int: 0 if not active, 1 if active.
 */
int TetrahedronGraphNode::isActive(TetrahedronGraphNode* b) {

  TetrahedronGraphEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->vertex == b) {
      return tempEdge->active;
    }
    tempEdge = vertices->getNext();
  }

  return 0;
}


/**
 * DEBUG PROC. Printout of node state.
 */
int TetrahedronGraphNode::print() {

  return 1;
}


/**
 * Add a vertex with an edge to this node.
 * @param b TetrahedronGraphNode*: node to add.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::addVertex(TetrahedronGraphNode* b) {

  TetrahedronGraphEdge* edge = new TetrahedronGraphEdge(b);
  vertices->push(edge);
  vertexCount++;
  if (isActive() && b->isActive()) {
    activeVertexCount++;
  }
  else {
    edge->active = 0;
  }

  return 1;
}


/**
 * Mark this node.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::mark() {

  marked = 1;

  return 1;
}


/**
 * Unmark this node.
 * @return int: 0 on fail, 1 on success.
 */
int TetrahedronGraphNode::unmark() {

  marked = 0;

  return 1;
}


/**
 * Check whether this node is marked or not.
 * @return int: 0 if not active, 1 if active.
 */
int TetrahedronGraphNode::isMarked() {

  return marked;
}
