
#include "triangleEdgeTree.h"


/** Constructor. */
TriangleEdgeTree::TriangleEdgeTree(TriangleEdge* e) :
  edge(e), child1(0), child2(0) {

  edgeList = new GenericList<TriangleGraphNode*>();

  return;
}

TriangleEdgeTree::TriangleEdgeTree(TriangleGraphNode* n) {

  Coordinate3D* coord1 = n->getCoordinate3D(0);
  Coordinate3D* coord2 = n->getCoordinate3D(1);
  Coordinate3D* coord3 = n->getCoordinate3D(2);

  if ( coord1->isGreaterThan(coord2) == 1 ||
       coord2->isGreaterThan(coord3) == 1 ) {
    printf("Error: TriangleEdgeTree::TriangleEdgeTree - coordinates out of order\n");
    return;
  }

  // Set edge to the middle edge of the triangle
  edge = new TriangleEdge(coord1,coord3);
  TriangleEdge* tempEdge1 = new TriangleEdge(coord1,coord2);
  TriangleEdge* tempEdge2 = new TriangleEdge(coord2,coord3);

  edgeList = new GenericList<TriangleGraphNode*>();
  edgeList->push(n);

  child1 = new TriangleEdgeTree(tempEdge1);
  child1->insert(tempEdge1,n);

  child2 = new TriangleEdgeTree(tempEdge2);
  child2->insert(tempEdge2,n);

  return;
}


/** Destructor. */
TriangleEdgeTree::~TriangleEdgeTree() {

  delete child1;
  delete child2;
  delete edge;
  delete edgeList;

  return;
}


/**
 * Add a node to the tree. Look at the 3 edges of the triangle.  For each edge, store a link to the triangle at the corresponding node in the tree.
 * @param node TriangleGraphNode*: node to add.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleEdgeTree::insert(TriangleGraphNode* node) {

  Coordinate3D* coord1 = node->getCoordinate3D(0);
  Coordinate3D* coord2 = node->getCoordinate3D(1);
  Coordinate3D* coord3 = node->getCoordinate3D(2);

  if (coord1->isGreaterThan(coord2) == 1 ||
      coord2->isGreaterThan(coord3) == 1) {
    printf("Error: TriangleEdgeTree::TriangleEdgeTree - coordinates out of order\n");
    return 0;
  }

  // Send edges off with node
  TriangleEdge* tempEdge1 = new TriangleEdge(coord1,coord2);
  TriangleEdge* tempEdge2 = new TriangleEdge(coord1,coord3);
  TriangleEdge* tempEdge3 = new TriangleEdge(coord2,coord3);

  int result = 0;
  result += insert(tempEdge1, node);
  result += insert(tempEdge2, node);
  result += insert(tempEdge3, node);

  if (result < 3) {
    return 0;
  }

  return 1;
}


/**
 * Add a node to the tree. Look at the 3 edges of the triangle.  For each edge, store a link to the triangle at the corresponding node in the tree.
 * @param edge TriangleEdge*: edge of triangle.
 * @param triangle TriangleGraphNode*: triangle to add.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleEdgeTree::insert(TriangleEdge* e, TriangleGraphNode* triangle) {

  if (edge->equals(e) == 1) {
    edgeList->push(triangle);
  }
  if (edge->isGreaterThan(e) == 1) {
    if (child1 == 0) {
      child1 = new TriangleEdgeTree(e);
    }
    return child1->insert(e, triangle);
  }
  if (edge->isLessThan(e) == 1) {
    if (child2 == 0) {
      child2 = new TriangleEdgeTree(e);
    }
    return child2->insert(e, triangle);
  }

  return 0;
}


/**
 * Connect the nodes that correspond to triangles sharing a side.
 * @param GenericList<IntPair*>*: list of TriangleNode id pairs for edge creation.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleEdgeTree::createEdges(GenericList<IntPair*>* idList) {

  TriangleGraphNode* tempNode1 = 0;
  TriangleGraphNode* tempNode2 = 0;
  TriangleGraphNode* tempNode3 = 0;
  int id1 = 0;
  int id2 = 0;

  // XXX - for a triangle, only add edges to the (max of 6) triangles that are closest, have the smallest angles to the given triangle for a specified side
  // XXX - for the above purpose, need to radially sort the edgeList
  sortEdgeList();

  //GenericList<IntPair*>* idList = new GenericList<IntPair*>();
  // tempEdgeList: store edgeList elements for later retrieval.
  GenericList<TriangleGraphNode*>* tempEdgeList = edgeList->clone();

  // Create IntPairs for each triangle pair sharing the current edge.
  /*
    while (tempEdgeList->getLength() > 0) {
    tempNode1 = tempEdgeList->front();
    //tempEdgeList->push(tempNode1);
    id1 = tempNode1->getId();
    tempNode2 = tempEdgeList->getFirst();
    while (tempNode2 != 0) {
    id2 = tempNode2->getId();
    //graph->addEdge(id1,id2);
    IntPair* idPair = new IntPair();
    idPair->x = id1;
    idPair->y = id2;
    idList->push(idPair);
    tempNode2 = tempEdgeList->getNext();
    }
    }
  */
  
  // XXX - DEBUG SORTED EDGELIST PRINTOUT
  printf("  Sorted edgeList:");
  tempNode1 = tempEdgeList->getFirst();
  while (tempNode1 != 0) {
    printf(" %d",tempNode1->getId());
    tempNode1 = tempEdgeList->getNext();
  }
  printf("\n");


  // Don't repair first and second elements of a 2-element list.
  int longList = 0;
  if (tempEdgeList->getLength() > 2) {
    longList = 1;
    tempNode3 = tempEdgeList->getFirst();
  }

  while (tempEdgeList->getLength() > 1) {
    tempNode1 = tempEdgeList->front();
    id1 = tempNode1->getId();
    tempNode2 = tempEdgeList->getFirst();
    id2 = tempNode2->getId();
    IntPair* idPair = new IntPair();
    idPair->x = id1;
    idPair->y = id2;
    idList->push(idPair);
  }

  // Pair corresponding to first and last elements of the original list.
  // Don't repair first and second elements of a 2-element list.
  tempNode1 = tempEdgeList->front();
  if (longList == 1) {
    id1 = tempNode3->getId();
    id2 = tempNode1->getId();
    IntPair* idPair = new IntPair();
    idPair->x = id1;
    idPair->y = id2;
    idList->push(idPair);
  }
  delete tempEdgeList;
  
  int result = 0;
  if (child1 != 0) {
    result = child1->createEdges(idList);
    if (result == 0) {
      printf("Error: TriangleEdgeTree::createEdges: child1 Missing the creation of edges\n");
    }
  }
  if (child2 != 0) {
    result = child2->createEdges(idList);
    if (result == 0) {
      printf("Error: TriangleEdgeTree::createEdges: child2 Missing the creation of edges\n");
    }
  }

  return 1;
}


/**
 * Radially ort the TriangleGraphNodes in the edgeList.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleEdgeTree::sortEdgeList() {

  //printf("->TriangleEdgeTree::sortEdgeList\n");

  // Find smallest opposing Coordinate3D, consider edge a vector from smaller to larger Coordinate3D, order TriangleGraphNodes by right-handed rotation about the edge
  Coordinate3D* edgeCoord1 = edge->coord1;
  Coordinate3D* edgeCoord2 = edge->coord2;

  //printf("  Hey1\n");
  // Find all opposing coordinates (triangle points not in the shared side).
  // Move first coordinate of side to the origin.
  // Move all other coordinates accordingly.
  // Changed over to "new" Coordinate3Ds.
  GenericList<Coordinate3D*>* coordList = new GenericList<Coordinate3D*>();
  //printf("    Hey1\n");
  TriangleGraphNode* tempTriangle = edgeList->getFirst();
  //printf("    Hey2\n");

  //printf("  edgeCoord1: (%3.4f,%3.4f,%3.4f)\n",edgeCoord1->getX(),edgeCoord1->getY(),edgeCoord1->getZ());
  //printf("  edgeCoord2: (%3.4f,%3.4f,%3.4f)\n",edgeCoord2->getX(),edgeCoord2->getY(),edgeCoord2->getZ());

  while (tempTriangle != 0) {
    //printf("    Yo\n");

    //printf("  triangle coordinate: (%3.4f,%3.4f,%3.4f)\n",tempTriangle->getCoordinate3D(0)->getX(),tempTriangle->getCoordinate3D(0)->getY(),tempTriangle->getCoordinate3D(0)->getZ());
    //printf("  triangle coordinate: (%3.4f,%3.4f,%3.4f)\n",tempTriangle->getCoordinate3D(1)->getX(),tempTriangle->getCoordinate3D(1)->getY(),tempTriangle->getCoordinate3D(1)->getZ());
    //printf("  triangle coordinate: (%3.4f,%3.4f,%3.4f)\n",tempTriangle->getCoordinate3D(2)->getX(),tempTriangle->getCoordinate3D(2)->getY(),tempTriangle->getCoordinate3D(2)->getZ());

    if (tempTriangle->getCoordinate3D(0)->equals(edgeCoord1) == 0 &&
	tempTriangle->getCoordinate3D(0)->equals(edgeCoord2) == 0 ) {
      //printf("    Hey3 pushing\n");
      coordList->push(tempTriangle->getCoordinate3D(0)->minus(edgeCoord1));
    }
    else if (! (tempTriangle->getCoordinate3D(1)->equals(edgeCoord1) == 1  ||
		tempTriangle->getCoordinate3D(1)->equals(edgeCoord2) == 1 ) ) {
      //printf("    Hey4\n");
      coordList->push(tempTriangle->getCoordinate3D(1)->minus(edgeCoord1));
    }
    else if (! (tempTriangle->getCoordinate3D(2)->equals(edgeCoord1) == 1  ||
		tempTriangle->getCoordinate3D(2)->equals(edgeCoord2) == 1 ) ) {
      //printf("    Hey5\n");
      coordList->push(tempTriangle->getCoordinate3D(2)->minus(edgeCoord1));
    }
    else {
      printf("Error TriangleEdgeTree::sortEdgeList screwey triangle in edgeList\n");
    }
    //printf("    Hey6\n");
    tempTriangle = edgeList->getNext();
  }
  edgeCoord2 = edgeCoord2->minus(edgeCoord1);

  Coordinate3D* tempCoord = coordList->getFirst();
  /*printf("coordList:");
    while (tempCoord != 0) {
    printf(" (%3.4f,%3.4f,%3.4f)",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
    tempCoord = coordList->getNext();
    }
    printf("\n");
  */
  //printf("  Hey2\n");
  // Rotate second coordinate of side to lie on y/z plane
  // Move all other coordinates accordingly.
  float angle = 0.0;
  float pi = 4.0*atan(1.0);
  //printf("  pi = %3.3f\n",pi);
  if (edgeCoord2->getY() == 0.0) {
    if (edgeCoord2->getX() > 0.0) {
      angle = pi/2.0;
    }
    else {
      angle = -pi/2.0;
    }
  }
  else {
    angle = atan(edgeCoord2->getX() / edgeCoord2->getY());
    if (edgeCoord2->getY() < 0.0) {
      angle += pi;
    }
  }

  //printf("  Hey3\n");
  edgeCoord2->rotateZ(angle);
  tempCoord = coordList->getFirst();
  while (tempCoord != 0) {
    tempCoord->rotateZ(angle);
    tempCoord = coordList->getNext();
  }

  /*
    tempCoord = coordList->getFirst();
    printf("coordList:");
    while (tempCoord != 0) {
    printf(" (%3.4f,%3.4f,%3.4f)",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
    tempCoord = coordList->getNext();
    }
    printf("\n");
  */

  //printf("  Hey4\n");
  // Rotate second coordinate of side to lie on z axis.
  // Move all other coordinates accordingly.
  if (edgeCoord2->getZ() == 0.0) {
    if (edgeCoord2->getY() > 0.0) {
      angle = pi/2.0;
    }
    else {
      angle = -pi/2.0;
    }
  }
  else {
    angle = atan(edgeCoord2->getY() / edgeCoord2->getZ());
    if (edgeCoord2->getZ() < 0.0) {
      angle += pi;
    }
  }

  //printf("  Hey5\n");
  edgeCoord2->rotateX(angle);
  tempCoord = coordList->getFirst();
  while (tempCoord != 0) {
    tempCoord->rotateX(angle);
    tempCoord = coordList->getNext();
  }

  // Ignore all z-components of coordinates, projecting onto x/y plane.
  // Sort coordinates by counter-clockwise angle from x-axis.
  float tempAngle = 0.0;
  GenericList<float>* angleList = new GenericList<float>();
  //printf("coordList->getLength(): %d\n",coordList->getLength());
  tempCoord = coordList->getFirst();
  while (tempCoord != 0) {
    if (tempCoord->getX() == 0.0) {
      if (tempCoord->getY() > 0.0) {
	angleList->push(pi/2.0);
      }
      else {
	angleList->push(pi * (3.0/2.0));
      }
    }
    else {
      tempAngle = atan(tempCoord->getY() / tempCoord->getX());
      if (tempCoord->getX() > 0.0) {
	if (tempCoord->getY() < 0.0) {
	  angleList->push((2.0 * pi) + tempAngle);
	}
	// "else" case is the default.
	else {
	  angleList->push(tempAngle);
	}
      }
      else {
	// Covers both y>0 and y<0 cases.
	angleList->push(pi + tempAngle);
      }
    }
    tempCoord = coordList->getNext();
  }

  // XXX DEBUG PRINTOUT OF EDGELIST, COORDLIST, AND ANGLELIST
  TriangleGraphNode* tempEdge = edgeList->getFirst();
  /*
    printf("edgeList:");
    while (tempEdge != 0) {
    printf(" %d",tempEdge->getId());
    tempEdge = edgeList->getNext();
    }
    printf("\n");
    tempEdge = edgeList->getFirst();
    printf("edgeCoords:\n");
    while (tempEdge != 0) {
    printf("   (%3.4f,%3.4f,%3.4f)\n",tempEdge->getCoordinate3D(0)->getX(),tempEdge->getCoordinate3D(0)->getY(),tempEdge->getCoordinate3D(0)->getZ());
    printf("   (%3.4f,%3.4f,%3.4f)\n",tempEdge->getCoordinate3D(1)->getX(),tempEdge->getCoordinate3D(1)->getY(),tempEdge->getCoordinate3D(1)->getZ());
    printf("   (%3.4f,%3.4f,%3.4f)\n",tempEdge->getCoordinate3D(2)->getX(),tempEdge->getCoordinate3D(2)->getY(),tempEdge->getCoordinate3D(2)->getZ());
    tempEdge = edgeList->getNext();
    }
    tempCoord = coordList->getFirst();
    printf("coordList:");
    while (tempCoord != 0) {
    printf(" (%3.4f,%3.4f,%3.4f)",tempCoord->getX(),tempCoord->getY(),tempCoord->getZ());
    tempCoord = coordList->getNext();
    }
    printf("\n");
    tempAngle = angleList->getFirst();
    printf("angleList:");
    while (tempAngle != 0) {
    printf(" %3.4f",tempAngle);
    tempAngle = angleList->getNext();
    }
    printf("\n");
  */

  GenericList<TriangleGraphNode*>* tempEdgeList = new GenericList<TriangleGraphNode*>();
  //float previousAngle = 0.0;
  float currentLowAngle = 0.0;
  int currentLowAngleIndex = 0;
  //TriangleGraphNode* tempEdge = 0;
  //int listLen = edgeList->getLength();
  //printf("edgeList->getLength(): %d, angleList->getLength(): %d\n",edgeList->getLength(),angleList->getLength());
  int i=0;
  while (edgeList->getLength() > 0 &&
	 angleList->getLength() > 0) {
    i=0;
    tempEdge = edgeList->getFirst();
    tempAngle = angleList->getFirst();
    currentLowAngleIndex = i;
    currentLowAngle = tempAngle;
    while (tempEdge != 0 && tempAngle != 0) {
      if (tempAngle < currentLowAngle) {
	currentLowAngleIndex = i;
	currentLowAngle = tempAngle;
      }
      i++;
      tempEdge = edgeList->getNext();
      tempAngle = angleList->getNext();
    }
    //printf("edgeList ");
    //edgeList->printList();
    //printf("angleList ");
    //angleList->printList();
    tempEdgeList->push(edgeList->removeElement(currentLowAngleIndex));
    angleList->removeElement(currentLowAngleIndex);
    //printf("tempEdgeList ");
    //tempEdgeList->printList();
  }
  delete edgeList;
  edgeList = tempEdgeList;

  // Delete "new" Coordinate3Ds.
  delete edgeCoord2;
  while (coordList->getLength() > 0) {
    delete coordList->front();
  }
  delete coordList;
  delete angleList;

  //printf("<-TriangleEdgeTree::sortEdgeList\n");

  return 1;
}


/**
 * Connect the nodes that correspond to triangles sharing a side.
 * @param graph TriangleGraph*: graph that adds the edges.
 * @return int: 0 on fail, 1 on success.
 */
/*
int TriangleEdgeTree::createEdges2(TriangleGraph* graph) {

  TriangleGraphNode* tempNode1 = 0;
  TriangleGraphNode* tempNode2 = 0;
  int id1 = 0;
  int id2 = 0;

  while (edgeList->getLength() > 0) {
    tempNode1 = edgeList->front();
    id1 = tempNode1->getId();
    tempNode2 = edgeList->getFirst();
    while (tempNode2 != 0) {
      id2 = tempNode2->getId();
      graph->addEdge(id1,id2);
      tempNode2 = edgeList->getNext();
    }
  }

  int result = 0;
  if (child1 != 0) {
    result = child1->createEdges(graph);
    if (result == 0) {
      printf("Error: TriangleEdgeTree::createEdges: child1 Missing the creation of edges\n");
    }
  }
  if (child2 != 0) {
    result = child2->createEdges(graph);
    if (result == 0) {
      printf("Error: TriangleEdgeTree::createEdges: child2 Missing the creation of edges\n");
    }
  }

  return 1;
}
*/
