
#include "triangleGraphNode.h"

/** Constructor. */
TriangleGraphNode::TriangleGraphNode(BaseVertexNode* node1,
				     BaseVertexNode* node2,
				     BaseVertexNode* node3) :
  group(0), id(-1), active(1), marked(0), type(DOUBLE),
     vertexCount(0), activeVertexCount(0)  {

  //coord = new Coordinate3D(c->coords[0],c->coords[1],c->coords[2]);
  
  triangle = new TriangleNode(node1, node2, node3);
  vertices = new GenericList<TriangleGraphEdge*>();

  return;
}

/** Destructor. */
TriangleGraphNode::~TriangleGraphNode() {

  delete triangle;
  delete vertices;

  return;
}


/**
 * Get first node.
 * @return TriangleGraphNode*: first node, 0 on fail.
 */
TriangleGraphNode* TriangleGraphNode::getFirstVertex() {

  TriangleGraphEdge* tempEdge = vertices->getFirst();

  if (tempEdge == 0) {
    return 0;
  }

  return tempEdge->vertex;
}


/**
 * Get next node.
 * @return TriangleGraphNode*: next node, 0 on fail.
 */
TriangleGraphNode* TriangleGraphNode::getNextVertex() {

  TriangleGraphEdge* tempEdge = vertices->getNext();
  
  if (tempEdge == 0) {
    return 0;
  }

  return tempEdge->vertex;
}


/**
 * Get the 3D coordinates.
   * @param i int: index 0, 1, or 2.
   * @return Coordinate3D*: triangle corner.
 */
Coordinate3D* TriangleGraphNode::getCoordinate3D(int i) {

  if (i == 0) {
    return triangle->node1->getCoordinate3D();
  }
  if (i == 1) {
    return triangle->node2->getCoordinate3D();
  }
  if (i == 2) {
    return triangle->node3->getCoordinate3D();
  }

  printf("Error TriangleGraphNode::getCoordinate3D - index %d out of bounds\n", i);

  return 0;
}


/**
 * Get the value of the node.
 * @param i int: index 0, 1, or 2.
 * @return float: value.
 */
float TriangleGraphNode::getValue(int i) {

  if (i == 0) {
    return triangle->node1->getValue();
  }
  if (i == 1) {
    return triangle->node2->getValue();
  }
  if (i == 2) {
    return triangle->node3->getValue();
  }

  printf("Error TriangleGraphNode::getValue - index %d out of bounds\n", i);

  return 0;
}


/**
 * Get ID.
 * @return int: ID.
 */
int TriangleGraphNode::getId() {

  return id;
}


/**
 * Set ID.
 * @param i int: ID.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::setId(int i) {

  id = i;

  return 1;
}


/**
 * Get group ID.
 * @return GroupId*: group ID.
 */
GroupId* TriangleGraphNode::getGroupId() {

  return group;
}


/**
 * Set group ID.
 * @param gId GroupId*: group ID.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::setGroupId(GroupId* gId) {

  group = gId;

  return 1;
}


/**
 * Get type.
 * @return int: type based on enum.
 */
int TriangleGraphNode::getType() {

  return type;
}


/**
 * Set type.
 * @param t int: type based on enum.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::setType(int t) {
  
  //printf("->TriangleGraphNode::setType\n");

  type = t;

  //printf("<-TriangleGraphNode::setType\n");

  return 1;
}


/**
 * Get vertex count.
 * @return int: vertex count.
 */
int TriangleGraphNode::getVertexCount() {

  return vertexCount;
}


/**
 * Get active vertex count.
 * @return int: active vertex count.
 */
int TriangleGraphNode::getActiveVertexCount() {

  return activeVertexCount;
}


/**
 * Get the surface area of the triangle.  INTERNAL triangles have no surface area, EXTERNAL triangles contribute one side, and DOUBLE triangles contribute both sides.
 * @return float: surface area.
 */
float TriangleGraphNode::getSurfaceArea() {

  if (getType() == INTERNAL) {
    return 0.0;
  }

  Coordinate3D* x = triangle->node1->getCoordinate3D();
  Coordinate3D* y = triangle->node2->getCoordinate3D();
  Coordinate3D* z = triangle->node3->getCoordinate3D();
  float a = x->getDistanceTo(y);
  float b = y->getDistanceTo(z);
  float c = z->getDistanceTo(x);
  float surfaceArea = sqrt((a + b + c) * (b + c - a) * (a + c - b) * (a + b - c))/4;

  if (getType() == EXTERNAL) {
    return surfaceArea;
  }
  
  if (getType() == DOUBLE) {
    surfaceArea *= 2.0;
    return surfaceArea;
  }

  printf("Error TriangleGraphNode::getSurfaceArea invalid type\n");

  return 0.0;
}


/**
 * Activate this node.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::activate() {

  active = 1;

  return 1;
}


/**
 * Deactivate this node.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::deactivate() {

  //printf("  deactivate: %d\n",getId());
  active = 0;

  TriangleGraphEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->active == 1) {
      tempEdge->vertex->deactivateEdge(this);
      tempEdge->active = 0;
      activeVertexCount--;
    }
    tempEdge = vertices->getNext();
  }

  return 1;
}


/**
 * Activate the edge leading to the specified node.
 * @param b TriangleGraphNode*: node the edge connects to.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::activateEdge(TriangleGraphNode* b) {

  if (isActive() && b->isActive()) {
    TriangleGraphEdge* tempEdge = vertices->getFirst();
    while (tempEdge != 0) {
      if (tempEdge->vertex == b) {
	tempEdge->active = 1;
	activeVertexCount++;
	return 1;
      }
      tempEdge = vertices->getNext();
    }
  }

  return 1;
}


/**
 * Deactivate the edge leading to the specified node.
 * @param b TriangleGraphNode*: node the edge connects to.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::deactivateEdge(TriangleGraphNode* b) {

  TriangleGraphEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->vertex == b) {
      tempEdge->active = 0;
      activeVertexCount--;
      return 1;
    }
    tempEdge = vertices->getNext();
  }

  return 0;
}


/**
 * Check whether this node is active or not.
 * @return int: 0 if not active, 1 if active.
 */
int TriangleGraphNode::isActive() {

  return active;
}


/**
 * Check whether the edge leading to the specified node is active.
 * @param b TriangleGraphNode*: node the edge connects to.
 * @return int: 0 if not active, 1 if active.
 */
int TriangleGraphNode::isActive(TriangleGraphNode* b) {

  TriangleGraphEdge* tempEdge = vertices->getFirst();
  while (tempEdge != 0) {
    if (tempEdge->vertex == b) {
      return tempEdge->active;
    }
    tempEdge = vertices->getNext();
  }

  return 0;
}


/**
 * Check whether this triangle is greater than a given triangle.
 * @param b TriangleGraphNode*: comparison node.
 * @return int: 0 if less than or equal to, 1 if greater.
 */
int TriangleGraphNode::isGreaterThan(TriangleGraphNode* b) {

  Coordinate3D* coord1 = getCoordinate3D(0);
  Coordinate3D* coord2 = b->getCoordinate3D(0);

  if (coord1->isGreaterThan(coord2)) {
    return 1;
  }
  if (coord1->isLessThan(coord2)) {
    return 0;
  }

  coord1 = getCoordinate3D(1);
  coord2 = b->getCoordinate3D(1);

  if (coord1->isGreaterThan(coord2)) {
    return 1;
  }
  if (coord1->isLessThan(coord2)) {
    return 0;
  }

  coord1 = getCoordinate3D(2);
  coord2 = b->getCoordinate3D(2);

  if (coord1->isGreaterThan(coord2)) {
    return 1;
  }
  if (coord1->isLessThan(coord2)) {
    return 0;
  }

  return 0;
}


/**
 * Check whether this triangle is less than a given triangle.
 * @param b TriangleGraphNode*: comparison node.
 * @return int: 0 if greater than or equal to, 1 if less.
 */
int TriangleGraphNode::isLessThan(TriangleGraphNode* b) {

  Coordinate3D* coord1 = getCoordinate3D(0);
  Coordinate3D* coord2 = b->getCoordinate3D(0);

  if (coord1->isLessThan(coord2)) {
    return 1;
  }
  if (coord1->isGreaterThan(coord2)) {
    return 0;
  }

  coord1 = getCoordinate3D(1);
  coord2 = b->getCoordinate3D(1);

  if (coord1->isLessThan(coord2)) {
    return 1;
  }
  if (coord1->isGreaterThan(coord2)) {
    return 0;
  }

  coord1 = getCoordinate3D(2);
  coord2 = b->getCoordinate3D(2);

  if (coord1->isLessThan(coord2)) {
    return 1;
  }
  if (coord1->isGreaterThan(coord2)) {
    return 0;
  }

  return 0;
}


/**
 * Check whether this triangle contains the given Coordinate3D.
 * @param coord Coordinate3D*: coordinate to check for.
 * @return int: 0 if triangle does not have coord, 1 if it does.
 */
int TriangleGraphNode::hasCoordinate3D(Coordinate3D* coord) {

  if ( coord->equals(getCoordinate3D(0)) ||
       coord->equals(getCoordinate3D(1)) ||
       coord->equals(getCoordinate3D(2)) ) {
    return 1;
  }

  return 0;
}


/**
 * DEBUG PROC. Printout of node state.
 */
int TriangleGraphNode::print() {

  int g = 0;
  if (group != 0) {
    g = group->getId();
  }

  printf("TriangleGraphNode:\n");
  Coordinate3D* coord = triangle->node1->getCoordinate3D();
  printf("  coords: (%3.2f,%3.2f,%3.2f)\n",
	 coord->getX(), coord->getY(), coord->getZ());
  coord = triangle->node2->getCoordinate3D();
  printf("          (%3.2f,%3.2f,%3.2f)\n",
	 coord->getX(), coord->getY(), coord->getZ());
  coord = triangle->node3->getCoordinate3D();
  printf("          (%3.2f,%3.2f,%3.2f)\n",
	 coord->getX(), coord->getY(), coord->getZ());
  printf("  group: %d\n", g);
  printf("  id: %d\n", id);
  printf("  active: %d\n", active);
  printf("  marked: %d\n", marked);
  printf("  vertexCount: %d\n", vertexCount);
  printf("  activeVertexCount: %d\n", activeVertexCount);
  printf("  vertices: ");
  TriangleGraphEdge* tempEdge = vertices->getFirst();
  do {
    if ( tempEdge != 0) {
      printf("%d ",tempEdge->vertex->getId());
      tempEdge = vertices->getNext();
    }
  } while ( tempEdge != 0 );
  printf("\n");
  printf("  active vertices: ");
  tempEdge = vertices->getFirst();
  do {
    if ( tempEdge != 0) {
      if ( tempEdge->active == 1) {
	printf("%d ",tempEdge->vertex->getId());
      }
      tempEdge = vertices->getNext();
    }
  } while ( tempEdge != 0 );
  printf("\n");

  return 1;
}


int TriangleGraphNode::print2() {

  printf("Triangle: %d, (%d, %d, %d)\n",
	 getId(),
	 triangle->node1->getId(),
	 triangle->node2->getId(),
	 triangle->node3->getId());

  return 1;
}


int TriangleGraphNode::addVertex(TriangleGraphNode* b) {

  TriangleGraphEdge* edge = new TriangleGraphEdge(b);
  vertices->push(edge);
  vertexCount++;
  if (isActive() && b->isActive()) {
    activeVertexCount++;
  }
  else {
    edge->active = 0;
  }

  return 1;
}


/**
 * Mark this node.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::mark() {

  marked = 1;

  return 1;
}


/**
 * Unmark this node.
 * @return int: 0 on fail, 1 on success.
 */
int TriangleGraphNode::unmark() {

  marked = 0;

  return 1;
}


/**
 * Check whether this node is marked or not.
 * @return int: 0 if not active, 1 if active.
 */
int TriangleGraphNode::isMarked() {

  return marked;
}
