
#ifndef TRIANGLEGRAPHNODE_H
#define TRIANGLEGRAPHNODE_H

#include "baseVertexNode.h"
#include "coordinate3D.h"
#include "genericList.h"
#include "groupId.c"

#include <math.h>
#include <stdio.h>


struct TriangleNode {
  // Nodes in order from least to greatest.
  BaseVertexNode* node1;
  BaseVertexNode* node2;
  BaseVertexNode* node3;
  
  TriangleNode(BaseVertexNode* x,
	       BaseVertexNode* y,
	       BaseVertexNode* z) {

    Coordinate3D* coord1 = x->getCoordinate3D();
    Coordinate3D* coord2 = y->getCoordinate3D();
    Coordinate3D* coord3 = z->getCoordinate3D();
    
    if (coord1->isLessThan(coord2)) {
      if (coord1->isLessThan(coord3)) {
	node1 = x;
	if (coord2->isLessThan(coord3)) {
	  node2 = y;
	  node3 = z;
	}
	else {
	  node2 = z;
	  node3 = y;
	}
      }
      else {
	node2 = x;
	if (coord2->isLessThan(coord3)) {
	  printf("Error: TriangleNode - coord2 < coord3 Constructor is screwed\n");
	}
	else {
	  node1 = z;
	  node3 = y;
	}
      }
    }
    else {
      if (coord1->isLessThan(coord3)) {
	node2 = x;
	if (coord2->isLessThan(coord3)) {
	  node1 = y;
	  node3 = z;
	}
	else {
	  printf("Error: TriangleNode - coord2 > coord3 Constructor is screwed\n");
	}
      }
      else {
	node3 = x;
	if (coord2->isLessThan(coord3)) {
	  node1 = y;
	  node2 = z;
	}
	else {
	  node1 = z;
	  node2 = y;
	}
      }
    }

    return;
  }

};


/**
 * TriangleGraphNode
 */
class TriangleGraphNode {

  /**
   * Edges for TriangleGraphNode.
   */
  struct TriangleGraphEdge {
    TriangleGraphNode* vertex;
    int active;
    
    TriangleGraphEdge(TriangleGraphNode* v) :
      vertex(v), active(1) {}
  };

 public:

  /**
   * Enum for setting the type.
   */
  enum{INTERNAL, EXTERNAL, DOUBLE};
  
  
  /** Constructor. */
  TriangleGraphNode(BaseVertexNode* node1,
		    BaseVertexNode* node2,
		    BaseVertexNode* node3);

  /** Destructor. */
  ~TriangleGraphNode();

  
  /**
   * Get first node.
   * @return TriangleGraphNode*: first node, 0 on fail.
   */
  TriangleGraphNode* getFirstVertex();

  /**
   * Get next node.
   * @return TriangleGraphNode*: next node, 0 on fail.
   */
  TriangleGraphNode* getNextVertex();

  /**
   * Get the 3D coordinates.
   * @param i int: index 0, 1, or 2.
   * @return Coordinate3D*: triangle corner.
   */
  Coordinate3D* getCoordinate3D(int i);

  /**
   * Get the value of the node.
   * @param i int: index 0, 1, or 2.
   * @return float: value.
   */
  float getValue(int i);

  /**
   * Get ID.
   * @return int: ID.
   */
  int getId();

  /**
   * Set ID.
   * @param i int: ID.
   * @return int: 0 on fail, 1 on success.
   */
  int setId(int i);

  /**
   * Get group ID.
   * @return GroupId*: group ID.
   */
  GroupId* getGroupId();

  /**
   * Set group ID.
   * @param gId GroupId*: group ID.
   * @return int: 0 on fail, 1 on success.
   */
  int setGroupId(GroupId* gId);

  /**
   * Get type.
   * @return int: type based on enum.
   */
  int getType();

  /**
   * Set type.
   * @param t int: type based on enum.
   * @return int: 0 on fail, 1 on success.
   */
  int setType(int t);

  /**
   * Get vertex count.
   * @return int: vertex count.
   */
  int getVertexCount();

  /**
   * Get active vertex count.
   * @return int: active vertex count.
   */
  int getActiveVertexCount();

  /**
   * Get the surface area of the triangle.  INTERNAL triangles have no surface area, EXTERNAL triangles contribute one side, and DOUBLE triangles contribute both sides.
   * @return float: surface area.
   */
  float getSurfaceArea();

  /**
   * Activate this node.
   * @return int: 0 on fail, 1 on success.
   */
  int activate();

  /**
   * Deactivate this node.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivate();

  /**
   * Activate the edge leading to the specified node.
   * @param b TriangleGraphNode*: node the edge connects to.
   * @return int: 0 on fail, 1 on success.
   */
  int activateEdge(TriangleGraphNode* b);

  /**
   * Deactivate the edge leading to the specified node.
   * @param b TriangleGraphNode*: node the edge connects to.
   * @return int: 0 on fail, 1 on success.
   */
  int deactivateEdge(TriangleGraphNode* b);

  /**
   * Check whether this node is active or not.
   * @return int: 0 if not active, 1 if active.
   */
  int isActive();

  /**
   * Check whether the edge leading to the specified node is active.
   * @param b TriangleGraphNode*: node the edge connects to.
   * @return int: 0 if not active, 1 if active.
   */
  int isActive(TriangleGraphNode* b);

  /**
   * Check whether this triangle is greater than a given triangle.
   * @param b TriangleGraphNode*: comparison node.
   * @return int: 0 if less than or equal to, 1 if greater.
   */
  int isGreaterThan(TriangleGraphNode* b);

  /**
   * Check whether this triangle is less than a given triangle.
   * @param b TriangleGraphNode*: comparison node.
   * @return int: 0 if greater than or equal to, 1 if less.
   */
  int isLessThan(TriangleGraphNode* b);

  /**
   * Check whether this triangle contains the given Coordinate3D.
   * @param coord Coordinate3D*: coordinate to check for.
   * @return int: 0 if triangle does not have coord, 1 if it does.
   */
  int hasCoordinate3D(Coordinate3D* coord);

  /**
   * DEBUG PROC. Printout of node state.
   */
  int print();

  /**
   * DEBUG PROC. Printout of node state.
   */
  int print2();

  //protected:
  /**
   * Add a vertex with an edge to this node.
   * @param b TriangleGraphNode*: node to add.
   * @return int: 0 on fail, 1 on success.
   */
  int addVertex(TriangleGraphNode* b);
  
  /**
   * Mark this node.
   * @return int: 0 on fail, 1 on success.
   */
  int mark();

  /**
   * Unmark this node.
   * @return int: 0 on fail, 1 on success.
   */
  int unmark();

  /**
   * Check whether this node is marked or not.
   * @return int: 0 if not active, 1 if active.
   */
  int isMarked();


 private:
  TriangleNode* triangle;
  GenericList<TriangleGraphNode::TriangleGraphEdge*>* vertices;   // Connected through shared edges
  GroupId* group;
  int id;
  int active;
  int marked;
  int type;   // -1: unassigned, 0: internal, 1: external, 2: double-sided
  int vertexCount;
  int activeVertexCount;

};

#endif
